@@ -3,8 +3,12 @@
 #undef HAVE_GETTEXT
 #undef HAVE_LC_MESSAGES
 #undef HAVE_STPCPY
-#undef LOCALEDIR
-#undef W32CODECDIR
 
 #undef PACKAGE
 #undef VERSION
+#undef USE_GTK2
+
+#undef VMAID_BUILD_ENVIRONMENT
+
+#undef HAVE_I386_SET_LDT
+#undef USE_CODEC
