@@ -214,324 +214,562 @@ void avi_split_file(AviEdit *avi_edit,const gint pos)
 *                                                                             *
 ******************************************************************************/
 /*	AVIAVIե빽¤Τ
-	 file,ե̾
-	param,ȥ꡼ֹ
-	 type,streamtypeAUDIO/streamtypeVIDEO
-	 rate,졼
-	scale,
-	  RET,AVIե빽¤,NULL:顼										*/
-static AviFile *avi_open_avi(const gchar *file,const gint param,
-									guint32 *type,guint32 *rate,guint32 *scale)
+	    file,ե̾
+	   param,ȥ꡼ֹ
+	    type,streamtypeAUDIO/streamtypeVIDEO
+	    rate,졼
+	   scale,
+	priority,ͥ
+	language,
+	    name,̾
+	     RET,AVIե빽¤,NULL:顼									*/
+static AviFile *
+avi_open_avi (const gchar *file, const gint param,
+							guint32 *type, guint32 *rate, guint32 *scale,
+							guint16 *priority, guint16 *language, gchar **name)
 {
-	gint i;
-	guint32 size;
+	gchar *strn;
+	gint i, j;
+	guint32 id, size;
 	off_t offset;				/* AVIΥࡼӡ */
 	AviFile *avi_file;
-	AviIndexEntry avi_index;
+	AviBaseIndex abi;
+	AviBaseIndexEntry abe;
+	AviSuperIndex asi;
+	AviSuperIndexEntry ase;
+	AviIndexEntry aie;
 	AviStreamHeader *ash;
 	Chunk *ck;
 
-	avi_file=g_malloc(sizeof(AviFile));
+	avi_file = g_malloc0 (sizeof (AviFile));
 	/* ե򳫤 */
-	if ((avi_file->fd=open(file,O_RDONLY))==-1) {
-		g_free(avi_file);
+	avi_file->fd = open (file, O_RDONLY);
+	if (avi_file->fd == -1)
+	  {
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 #ifdef HAVE_FLOCK
-	if (flock(avi_file->fd,LOCK_SH)==-1) {
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (flock (avi_file->fd, LOCK_SH) != 0)
+	  {
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 #endif
 	/* եǥ꥿󥯥ݥ󥿤 */
-	ck=chunk_open(avi_file->fd);
+	ck = chunk_open (avi_file->fd);
 	/* RIFF󥯤 */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (!chunk_in (avi_file->fd, ck))
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 	/* եबhdrlLIST󥯤õ */
-	while (chunk_form(avi_file->fd)!=listtypeAVIHEADER)
-		if (!chunk_next(avi_file->fd,ck)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(avi_file);
+	while (chunk_form (avi_file->fd) != listtypeAVIHEADER)
+		if (!chunk_next (avi_file->fd, ck))
+		  {
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (avi_file);
 			return NULL;
-		}
+		  }
 	/* եबhdrlLIST󥯤(RIFF'AVI ' -> LIST'hdrl') */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (!chunk_in (avi_file->fd, ck))
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 	/* եबstrlLIST󥯤õ */
-	for (i=0;i<=param;i++)
+	for (i = 0;i <= param; i++)
 		do
-			if (!chunk_next(avi_file->fd,ck)) {
-				chunk_free(ck);
-				close(avi_file->fd);
-				g_free(avi_file);
+			if (!chunk_next (avi_file->fd, ck))
+			  {
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (avi_file);
 				return NULL;
-			}
-		while (chunk_form(avi_file->fd)!=listtypeSTREAMHEADER);
+			  }
+		while (chunk_form (avi_file->fd) != listtypeSTREAMHEADER);
 	/* եबstrlLIST󥯤
 									(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl') */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (!chunk_in (avi_file->fd, ck))
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 	/* strh󥯤õ(AviStreamHeader) */
-	while (chunk_id(avi_file->fd)!=ckidSTREAMHEADER)
-		if (!chunk_next(avi_file->fd,ck)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(avi_file);
+	while (chunk_id (avi_file->fd) != ckidSTREAMHEADER)
+		if (!chunk_next (avi_file->fd, ck))
+		  {
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (avi_file);
 			return NULL;
-		}
+		  }
 	/* strh󥯤Υ(AviStreamHeader) */
-	if ((size=chunk_size(avi_file->fd))==-1 || size<ASH_SIZE) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(avi_file);
+	if ((size = chunk_size (avi_file->fd)) == -1 || size < ASH_SIZE)
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
-	ash=g_malloc(ASH_SIZE);
+	  }
+	ash = g_malloc (ASH_SIZE);
 	/* strh󥯤(AVIStreamHeader)
 							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' -> strh) */
-	if (!chunk_in(avi_file->fd,ck)
+	if (!chunk_in (avi_file->fd, ck)
 		/* AviStreamHeaderɤ߹ */
-				|| read(avi_file->fd,ash,ASH_SIZE)!=ASH_SIZE
+				|| read (avi_file->fd, ash, ASH_SIZE) != ASH_SIZE
 		/*  */
-				|| (ash_get_type(ash)!=streamtypeAUDIO
-										&& ash_get_type(ash)!=streamtypeVIDEO)
+				|| (ash_get_type (ash) != streamtypeAUDIO
+									&& ash_get_type (ash) != streamtypeVIDEO)
 		/* strh󥯤Ф(AVIStreamHeader)
 							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' <- strh) */
-				|| !chunk_out(avi_file->fd,ck)
+				|| !chunk_out (avi_file->fd, ck)
 		/* եबstrlLIST󥯤Ф
 									(RIFF'AVI ' -> LIST'hdrl' <- LIST'strl') */
-				|| !chunk_out(avi_file->fd,ck)
+				|| !chunk_out (avi_file->fd, ck)
 		/* եबstrlLIST󥯤
 									(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl') */
-				|| !chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file);
+				|| !chunk_in (avi_file->fd, ck))
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (ash);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 	/* strf󥯤õ(BitmapInfoHeader)|(WaveFormatEx) */
-	while (chunk_id(avi_file->fd)!=ckidSTREAMFORMAT)
-		if (!chunk_next(avi_file->fd,ck)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file);
+	while (chunk_id (avi_file->fd) != ckidSTREAMFORMAT)
+		if (!chunk_next (avi_file->fd, ck))
+		  {
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file);
 			return NULL;
-		}
+		  }
 	/* strf󥯤Υ(BitmapInfoHeader)|(WaveFormatEx) */
-	if ((size=chunk_size(avi_file->fd))==-1) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file);
+	size = chunk_size (avi_file->fd);
+	if (size == -1)
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (ash);
+		g_free (avi_file);
 		return NULL;
-	}
-	if (ash_get_type(ash)==streamtypeVIDEO) {
-		if (size<BMIH_SIZE) {
-			chunk_free(ck);
+	  }
+	if (ash_get_type (ash) == streamtypeVIDEO)
+	  {
+		if (size < BMIH_SIZE)
+		  {
+			chunk_free (ck);
 			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file);
+			g_free (ash);
+			g_free (avi_file);
 			return NULL;
 		}
-		avi_file->bmih=g_malloc(size);
-		avi_file->wfx=NULL;
-	} else {
-		if (size<WFX_SIZE-sizeof(guint16)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file);
+		avi_file->bmih = g_malloc (size);
+	  }
+	else
+	  {
+		if (size < WFX_SIZE - sizeof (guint16))
+		  {
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file);
 			return NULL;
-		}
-		avi_file->wfx=g_malloc(MAX(size,WFX_SIZE));
-		wfx_set_size(avi_file->wfx,0);
-		avi_file->bmih=NULL;
-	}
+		  }
+		avi_file->wfx = g_malloc (MAX (size, WFX_SIZE));
+		wfx_set_size (avi_file->wfx, 0);
+	  }
 	/* strf󥯤(BitmapInfoHeader)|(WaveFormatEx)
 							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' -> strf) */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
+	if (!chunk_in (avi_file->fd, ck)
+		/* BitmapInfoHeader|WaveFormatExɤ߹ */
+		|| read (avi_file->fd, ash_get_type (ash) == streamtypeVIDEO
+			? (gpointer)avi_file->bmih : (gpointer)avi_file->wfx, size) != size
+		/* strf󥯤Ф(BitmapInfoHeader)|(WaveFormatEx)
+							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' <- strh) */
+		|| !chunk_out (avi_file->fd, ck)
+		/* եबstrlLIST󥯤Ф
+									(RIFF'AVI ' -> LIST'hdrl' <- LIST'strl') */
+		|| !chunk_out (avi_file->fd, ck)
+		/* եबstrlLIST󥯤
+									(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl') */
+		|| !chunk_in (avi_file->fd, ck))
+	  {
+		chunk_free (ck);
+		close (avi_file->fd);
+		g_free (ash);
+		g_free (avi_file->bmih);
+		g_free (avi_file->wfx);
+		g_free (avi_file);
 		return NULL;
-	}
-	/* BitmapInfoHeader|WaveFormatExɤ߹ */
-	if (read(avi_file->fd,ash_get_type(ash)==streamtypeVIDEO
-			?(gpointer)avi_file->bmih:(gpointer)avi_file->wfx,size)!=size) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-	/* 󥯥ݥ󥿤 */
-	if (!chunk_free(ck)) {
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-
-	/* ࡼӡ󥯤ذư */
-
-	/* եǥ꥿󥯥ݥ󥿤 */
-	ck=chunk_open(avi_file->fd);
-	/* RIFF󥯤 */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-	/* movi󥯤õ */
-	while (chunk_form(avi_file->fd)!=listtypeAVIMOVIE)
-		if (!chunk_next(avi_file->fd,ck)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file->bmih);
-			g_free(avi_file->wfx);
-			g_free(avi_file);
+	 }
+	strn = NULL;
+	if (name != NULL)
+	  {
+		/* strn󥯤õ */
+		while ((id = chunk_id (avi_file->fd)) != ckidSTREAMNAME)
+			if (!chunk_next (avi_file->fd, ck))
+				break;
+		if (id == ckidSTREAMNAME)
+		  {
+			/* strn󥯤Υ */
+			size = chunk_size (avi_file->fd);
+			if (size == -1)
+			  {
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (ash);
+				g_free (avi_file->bmih);
+				g_free (avi_file->wfx);
+				g_free (avi_file);
+				return NULL;
+			  }
+			strn = g_malloc0 (size + 1);
+			/* strn󥯤
+							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' -> strn) */
+			if (!chunk_in (avi_file->fd, ck)
+				/* strnɤ߹ */
+					|| read (avi_file->fd, strn, size) != size
+				/* strn󥯤Ф
+							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' <- strh) */
+					|| !chunk_out (avi_file->fd, ck))
+			  {
+				g_free (strn);
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (ash);
+				g_free (avi_file->bmih);
+				g_free (avi_file->wfx);
+				g_free (avi_file);
+				return NULL;
+			  }
+		  }
+		/* եबstrlLIST󥯤Ф
+									(RIFF'AVI ' -> LIST'hdrl' <- LIST'strl') */
+		if (!chunk_out (avi_file->fd, ck)
+			/* եबstrlLIST󥯤
+									(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl') */
+				|| !chunk_in (avi_file->fd, ck))
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
 			return NULL;
-		}
-	/* եΥեåȤ */
-	if ((offset=lseek(avi_file->fd,0,SEEK_CUR))==-1) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-	offset+=sizeof(guint32)+sizeof(guint32);/* moviΥեå */
-	/* 󥯥ݥ󥿤 */
-	if (!chunk_free(ck)) {
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
+		  }
+	  }
 
-	/* ǥå󥯤ذư */
+	/* ѡǥå󥯤ذư */
 
-	/* եǥ꥿󥯥ݥ󥿤 */
-	ck=chunk_open(avi_file->fd);
-	/* RIFF󥯤 */
-	if (!chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-	/* idx1󥯤õ */
-	while (chunk_id(avi_file->fd)!=ckidAVINEWINDEX)
-		if (!chunk_next(avi_file->fd,ck)) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file->bmih);
-			g_free(avi_file->wfx);
-			g_free(avi_file);
+	/* ѡǥå󥯤õ */
+	while ((id = chunk_id (avi_file->fd)) != ckidAVISUPERINDEX)
+		if (!chunk_next (avi_file->fd, ck))
+			break;
+	if (id == ckidAVISUPERINDEX)
+	  {
+		/* ѡǥå󥯤Υ */
+		size = chunk_size (avi_file->fd);
+		if (size == -1	/* ѡǥå󥯤
+							(RIFF'AVI ' -> LIST'hdrl' -> LIST'strl' -> indx) */
+						|| !chunk_in (avi_file->fd, ck)
+						/* indxɤ߹ */
+						|| read (avi_file->fd, &asi, ASI_SIZE) != ASI_SIZE
+						|| size < ASI_SIZE + asi_get_entries (&asi) * ASE_SIZE)
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
 			return NULL;
-		}
-	/* idx1󥯤Υ */
-	/* idx1󥯤(RIFF'AVI ' -> idx1) */
-	if ((size=chunk_size(avi_file->fd))==-1 || !chunk_in(avi_file->fd,ck)) {
-		chunk_free(ck);
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
-		return NULL;
-	}
-	/* ǥåɤ߹ */
-	avi_file->entries=0;
-	avi_file->index=NULL;
-	for (i=0;i<size/AIE_SIZE;i++) {
-		if (read(avi_file->fd,&avi_index,AIE_SIZE)!=AIE_SIZE) {
-			chunk_free(ck);
-			close(avi_file->fd);
-			g_free(ash);
-			g_free(avi_file->bmih);
-			g_free(avi_file->wfx);
-			g_free(avi_file);
-			return NULL;
-		}
-		if (((aie_get_ckid(&avi_index)&0xff)-'0')*10
-							+((aie_get_ckid(&avi_index)>>8)&0xff)-'0'==param) {
-			avi_file->index=g_realloc(avi_file->index,
-											(avi_file->entries+1)*AIE_SIZE);
-			avi_file->index[avi_file->entries].offset
-								=aie_get_offset(&avi_index)
-									+offset+sizeof(guint32)+sizeof(guint32);
-			avi_file->index[avi_file->entries].size=aie_get_length(&avi_index);
-			avi_file->index[avi_file->entries].flags=aie_get_flags(&avi_index);
-			avi_file->entries++;
-		}
-	}
+		  }
+		if (asi_get_longs (&asi) == 4
+							&& asi_get_sub_type (&asi) == AVI_INDEX_SUB_DEFAULT
+								&& asi_get_type (&asi) == AVI_INDEX_OF_INDEXES)
+			for (i = 0; i < asi_get_entries (&asi); i++)
+			  {
+				/* ѡǥåȥ꡼ɤ߹ */
+				if (read (avi_file->fd, &ase, ASE_SIZE) != ASE_SIZE)
+				  {
+					g_free (strn);
+					chunk_free (ck);
+					close (avi_file->fd);
+					g_free (ash);
+					g_free (avi_file->index);
+					g_free (avi_file);
+					return NULL;
+				  }
+				if (ase_get_offset (&ase) != 0)
+				  {
+					/* եΥեåȤ */
+					offset = lseek (avi_file->fd, 0, SEEK_CUR);
+					/* եΥեåȤꤹ */
+					if (offset == -1 || lseek (avi_file->fd,
+							ase_get_offset (&ase) + sizeof (guint32) * 2,
+																SEEK_SET) == -1
+							/* ix##ɤ߹ */
+							|| read (avi_file->fd, &abi, ABI_SIZE) != ABI_SIZE)
+					  {
+						g_free (strn);
+						chunk_free (ck);
+						close (avi_file->fd);
+						g_free (ash);
+						g_free (avi_file->index);
+						g_free (avi_file->bmih);
+						g_free (avi_file->wfx);
+						g_free (avi_file);
+						return NULL;
+					  }
+					if (abi_get_longs (&abi) == 2
+						&& asi_get_sub_type (&asi) == abi_get_sub_type (&abi)
+								&& abi_get_type (&abi) == AVI_INDEX_OF_CHUNKS)
+					  {
+						avi_file->index = g_realloc (avi_file->index,
+								(avi_file->entries + abi_get_entries (&abi))
+														* sizeof(AviIndex));
+						for (j = 0; j < abi_get_entries (&abi); j++)
+						  {
+							/* ѡǥåȥ꡼ɤ߹ */
+							if (read (avi_file->fd, &abe, ABE_SIZE)
+																!= ABE_SIZE)
+							  {
+								g_free (strn);
+								chunk_free (ck);
+								close (avi_file->fd);
+								g_free (ash);
+								g_free (avi_file->index);
+								g_free (avi_file->bmih);
+								g_free (avi_file->wfx);
+								g_free (avi_file);
+								return NULL;
+							  }
+							avi_file->index[avi_file->entries].offset
+									= abi_get_offset (&abi)
+													+ abe_get_offset (&abe);
+							avi_file->index[avi_file->entries].size
+									= (abe_get_size (&abe) & 0x7fffffff);
+							avi_file->index[avi_file->entries].flags
+									= (abe_get_size (&abe) & 0x80000000) == 0
+														? AVIIF_KEYFRAME : 0;
+							avi_file->entries++;
+						  }
+					  }
+					/* եΥեåȤꤹ */
+					if (lseek (avi_file->fd, offset, SEEK_SET) == -1)
+					  {
+						g_free (strn);
+						chunk_free (ck);
+						close (avi_file->fd);
+						g_free (ash);
+						g_free (avi_file->index);
+						g_free (avi_file->bmih);
+						g_free (avi_file->wfx);
+						g_free (avi_file);
+						return NULL;
+					  }
+				  }
+			  }
+	  }
+
 	/* 󥯥ݥ󥿤 */
-	if (!chunk_free(ck)) {
-		close(avi_file->fd);
-		g_free(ash);
-		g_free(avi_file->bmih);
-		g_free(avi_file->wfx);
-		g_free(avi_file);
+	if (!chunk_free (ck))
+	  {
+		g_free (strn);
+		close (avi_file->fd);
+		g_free (ash);
+		g_free (avi_file->index);
+		g_free (avi_file->bmih);
+		g_free (avi_file->wfx);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
+
+	if (avi_file->entries <= 0)
+	  {
+		/* ࡼӡ󥯤ذư */
+
+		/* եǥ꥿󥯥ݥ󥿤 */
+		ck = chunk_open (avi_file->fd);
+		/* RIFF󥯤 */
+		if (!chunk_in (avi_file->fd, ck))
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+		/* movi󥯤õ */
+		while (chunk_form (avi_file->fd) != listtypeAVIMOVIE)
+			if (!chunk_next (avi_file->fd, ck))
+			  {
+				g_free (strn);
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (ash);
+				g_free (avi_file->bmih);
+				g_free (avi_file->wfx);
+				g_free (avi_file);
+				return NULL;
+			}
+		/* եΥեåȤ */
+		offset = lseek (avi_file->fd, 0, SEEK_CUR);
+		if (offset == -1)
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+		offset += sizeof (guint32) + sizeof (guint32);/* moviΥեå */
+		/* 󥯥ݥ󥿤 */
+		if (!chunk_free (ck))
+		  {
+			g_free (strn);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+
+		/* ǥå󥯤ذư */
+
+		/* եǥ꥿󥯥ݥ󥿤 */
+		ck = chunk_open (avi_file->fd);
+		/* RIFF󥯤 */
+		if (!chunk_in (avi_file->fd, ck))
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+		/* idx1󥯤õ */
+		while (chunk_id (avi_file->fd) != ckidAVINEWINDEX)
+			if (!chunk_next (avi_file->fd, ck))
+			  {
+				g_free (strn);
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (ash);
+				g_free (avi_file->bmih);
+				g_free (avi_file->wfx);
+				g_free (avi_file);
+				return NULL;
+			}
+		/* idx1󥯤Υ */
+		/* idx1󥯤(RIFF'AVI ' -> idx1) */
+		size = chunk_size (avi_file->fd);
+		if (size == -1 || !chunk_in (avi_file->fd, ck))
+		  {
+			g_free (strn);
+			chunk_free (ck);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+		/* ǥåɤ߹ */
+		for (i = 0; i < size / AIE_SIZE; i++)
+		  {
+			if (read (avi_file->fd, &aie, AIE_SIZE) != AIE_SIZE)
+			  {
+				g_free (strn);
+				chunk_free (ck);
+				close (avi_file->fd);
+				g_free (ash);
+				g_free (avi_file->index);
+				g_free (avi_file->bmih);
+				g_free (avi_file->wfx);
+				g_free (avi_file);
+				return NULL;
+			  }
+			if (((aie_get_ckid (&aie) & 0xff) - '0') * 10
+						+ ((aie_get_ckid (&aie) >> 8) & 0xff) - '0' == param)
+			  {
+				avi_file->index = g_realloc (avi_file->index,
+								(avi_file->entries + 1) * sizeof (AviIndex));
+				avi_file->index[avi_file->entries].offset
+								= aie_get_offset (&aie)
+								+ offset + sizeof (guint32) + sizeof(guint32);
+				avi_file->index[avi_file->entries].size
+													= aie_get_length (&aie);
+				avi_file->index[avi_file->entries].flags
+													= aie_get_flags (&aie);
+				avi_file->entries++;
+			  }
+		  }
+		/* 󥯥ݥ󥿤 */
+		if (!chunk_free (ck))
+		  {
+			g_free (strn);
+			close (avi_file->fd);
+			g_free (ash);
+			g_free (avi_file->bmih);
+			g_free (avi_file->wfx);
+			g_free (avi_file);
+			return NULL;
+		  }
+	  }
 	/* ǥåȥإå */
-	if (ash_get_type(ash)==streamtypeVIDEO) {
-		avi_file->length=avi_file->entries;
-	} else {
-		avi_file->length=0;
-		for (i=0;i<avi_file->entries;i++)
-			avi_file->length+=avi_file->index[i].size
-										/wfx_get_block_align(avi_file->wfx);
-	}
-	avi_file->handler=ash_get_handler(ash);
+	if (ash_get_type (ash) == streamtypeVIDEO)
+	  {
+		avi_file->length = avi_file->entries;
+	  }
+	else
+	  {
+		avi_file->length = 0;
+		for (i = 0; i < avi_file->entries; i++)
+			avi_file->length += avi_file->index[i].size
+										/ wfx_get_block_align (avi_file->wfx);
+	  }
+	avi_file->handler = ash_get_handler (ash);
 	/*  */
-	if (type!=NULL)
-		*type=ash_get_type(ash);
-	if (rate!=NULL)
-		*rate=ash_get_rate(ash);
-	if (scale!=NULL)
-		*scale=ash_get_scale(ash);
-	g_free(ash);
-	/* ե */
-	avi_file->data=NULL;				/*  */
+	if (type != NULL)
+		*type = ash_get_type (ash);
+	if (rate != NULL)
+		*rate = ash_get_rate (ash);
+	if (scale != NULL)
+		*scale = ash_get_scale (ash);
+	if (priority != NULL)
+		*priority = ash_get_priority (ash);
+	if (language != NULL)
+		*language = ash_get_language (ash);
+	if (name != NULL)
+		*name = strn;
+	g_free (ash);
 	return avi_file;
 }
 
@@ -545,18 +783,19 @@ static AviFile *avi_open_bitmap(const gchar *file)
 	AviFile *avi_file;
 	BitmapInfoHeader *bmih0,*bmih1;
 
-	avi_file=g_malloc(sizeof(AviFile));
+	avi_file = g_malloc0 (sizeof (AviFile));
 	/* ե򳫤 */
 	if ((avi_file->fd=open(file,O_RDONLY))==-1) {
 		g_free(avi_file);
 		return NULL;
 	}
 #ifdef HAVE_FLOCK
-	if (flock(avi_file->fd,LOCK_SH)==-1) {
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (flock (avi_file->fd, LOCK_SH) != 0)
+	  {
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 #endif
 	/* ˥إåɤ߹ */
 	if (lseek(avi_file->fd,BMFH_SIZE,SEEK_SET)==-1) {
@@ -637,8 +876,6 @@ static AviFile *avi_open_bitmap(const gchar *file)
 	avi_file->fd=-1;					/* եǥ꥿ */
 	avi_file->length=1;
 	avi_file->entries=1;				/* ǥåο */
-	/* ǥͭ */
-	avi_file->wfx=NULL;					/* ꥸʥΥեޥå */
 	/* ǥå */
 	avi_file->index=g_malloc(sizeof(AviIndex));
 	avi_file->index[0].offset=0;
@@ -658,18 +895,19 @@ static AviFile *avi_open_wave(const gchar *file)
 	AviFile *avi_file;
 	Chunk *ck;
 
-	avi_file=g_malloc(sizeof(AviFile));
+	avi_file = g_malloc0 (sizeof (AviFile));
 	/* ե򳫤 */
 	if ((avi_file->fd=open(file,O_RDONLY))==-1) {
 		g_free(avi_file);
 		return NULL;
 	}
 #ifdef HAVE_FLOCK
-	if (flock(avi_file->fd,LOCK_SH)==-1) {
-		close(avi_file->fd);
-		g_free(avi_file);
+	if (flock (avi_file->fd, LOCK_SH) != 0)
+	  {
+		close (avi_file->fd);
+		g_free (avi_file);
 		return NULL;
-	}
+	  }
 #endif
 	/* եǥ꥿󥯥ݥ󥿤 */
 	ck=chunk_open(avi_file->fd);
@@ -765,12 +1003,8 @@ static AviFile *avi_open_wave(const gchar *file)
 		return NULL;
 	}
 	/* ե */
-	avi_file->data=NULL;				/*  */
 	avi_file->length=size/wfx_get_block_align(avi_file->wfx);
 	avi_file->entries=1;				/* ǥåο */
-	/* ӥǥͭ */
-	avi_file->handler=0;
-	avi_file->bmih=NULL;				/* ꥸʥΥեޥå */
 	/* ǥå */
 	avi_file->index=g_malloc(sizeof(AviIndex));
 	avi_file->index[0].offset=offset;
@@ -781,52 +1015,70 @@ static AviFile *avi_open_wave(const gchar *file)
 
 
 /*	AVIAVIե빽¤Τ
-	 file,ե̾
-	param,ȥ꡼ֹ
-	 type,streamtypeAUDIO/streamtypeVIDEO
-	 rate,졼
-	scale,
-	  RET,AVIե빽¤,NULL:顼										*/
-AviFile *avi_open_file(const gchar *file,const gint param,
-									guint32 *type,guint32 *rate,guint32 *scale)
+	    file,ե̾
+	   param,ȥ꡼ֹ
+	    type,streamtypeAUDIO/streamtypeVIDEO
+	    rate,졼
+	   scale,
+	priority,ͥ
+	language,
+	    name,̾
+	     RET,AVIե빽¤,NULL:顼									*/
+AviFile *
+avi_open_file (const gchar *file, const gint param,
+							guint32 *type, guint32 *rate, guint32 *scale,
+							guint16 *priority, guint16 *language, gchar **name)
 {
-	AviFile *avi_file=NULL;
+	AviFile *avi_file = NULL;
 
 	/* եηʬ */
-	switch (avi_file_type(file)) {
+	switch (avi_file_type (file))
+	  {
 		case 0:/* AVI */
-			avi_file=avi_open_avi(file,param,type,rate,scale);
+			avi_file = avi_open_avi (file, param, type, rate, scale,
+													priority, language, name);
 			break;
 		case 1:/* ӥåȥޥå */
-			avi_file=param==0?avi_open_bitmap(file):NULL;
-			if (avi_file!=NULL) {
-				if (type!=NULL)
-					*type=streamtypeVIDEO;
-				if (rate!=NULL)
-					*rate=15;
-				if (scale!=NULL)
-					*scale=1;
-			}
+			avi_file = param == 0 ? avi_open_bitmap (file) : NULL;
+			if (avi_file != NULL)
+			  {
+				if (type != NULL)
+					*type = streamtypeVIDEO;
+				if (rate != NULL)
+					*rate = 15;
+				if (scale != NULL)
+					*scale = 1;
+				if (priority != NULL)
+					*priority = 0;
+				if (language != NULL)
+					*language = 0;
+				if (name != NULL)
+					*name = g_strdup ("Video #1");
+			  }
 			break;
 		case 2:/* WAVEե */
-			avi_file=param==0?avi_open_wave(file):NULL;
-			if (avi_file!=NULL) {
-				if (type!=NULL)
-					*type=streamtypeAUDIO;
-				if (rate!=NULL)
-					*rate=wfx_get_average_bytes_per_sec(avi_file->wfx);
-				if (scale!=NULL)
-					*scale=wfx_get_block_align(avi_file->wfx);
-			}
-	}
-	if (avi_file!=NULL) {
-		avi_file->name=misc_get_full_path(file);	/* ե̾ */
-		avi_file->start=0;							/* ͭϰ */
-		avi_file->param=param;						/* ȥ꡼ֹ */
-		/* ꥹ */
-		avi_file->prev=NULL;
-		avi_file->next=NULL;
-	}
+			avi_file = param == 0 ? avi_open_wave (file) : NULL;
+			if (avi_file != NULL)
+			  {
+				if (type != NULL)
+					*type = streamtypeAUDIO;
+				if (rate != NULL)
+					*rate = wfx_get_average_bytes_per_sec (avi_file->wfx);
+				if (scale != NULL)
+					*scale = wfx_get_block_align (avi_file->wfx);
+				if (priority != NULL)
+					*priority = 0;
+				if (language != NULL)
+					*language = 0;
+				if (name != NULL)
+					*name = g_strdup ("Audio #1");
+			  }
+	  }
+	if (avi_file != NULL)
+	  {
+		avi_file->name = misc_get_full_path (file);	/* ե̾ */
+		avi_file->param = param;					/* ȥ꡼ֹ */
+	  }
 	return avi_file;
 }
 
