@@ -25,259 +25,106 @@
 #define __AVICORE_H__
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-
-#include <glib.h>
+#include "common.h"
 #include "bitmap.h"
-#include "chunk.h"
+#include "icm.h"
 #include "wave.h"
 
 
-/******************************************************************************
-*                                                                             *
-* ե빽¤                                                              *
-*                                                                             *
-******************************************************************************/
-#define AVIF_HASINDEX		0x00000010
-#define AVIF_MUSTUSEINDEX	0x00000020
-#define AVIF_ISINTERLEAVED	0x00000100
-#define AVIF_TRUSTCKTYPE	0x00000800
-#define AVIF_WASCAPTUREFILE	0x00010000
-#define AVIF_COPYRIGHTED	0x00020000
-#define AVISF_DISABLED		0x00000001
-#define AVIIF_LIST			0x00000001
-#define AVIIF_KEYFRAME		0x00000010
-#define AVIIF_NOTIME		0x00000100
-#define AVIIF_COMPUSE		0x0FFF0000
-#define streamtypeAUDIO 	make4cc('a','u','d','s')
-#define streamtypeVIDEO 	make4cc('v','i','d','s')
-
-
-#define AMH_SIZE 56
-#define RECT_SIZE 8
-#define ASH_SIZE 56
-#define AIE_SIZE 16
-
-
-#ifdef USE_GCC
-
-
-/* AVI Main File Header */
-typedef struct AviMainHeader_Tag {
-	guint32 micro_sec_per_frame __attribute__((packed));
-	guint32 max_bytes_per_sec __attribute__((packed));
-	guint32 padding_granularity __attribute__((packed));
-	guint32 flags __attribute__((packed));
-	guint32 total_frames __attribute__((packed));
-	guint32 initial_frames __attribute__((packed));
-	guint32 streams __attribute__((packed));
-	guint32 suggested_buffer_size __attribute__((packed));
-	guint32 width __attribute__((packed));
-	guint32 height __attribute__((packed));
-	guint32 reserved[4] __attribute__((packed));
-} AviMainHeader;
-/* Rectangle */
-typedef struct Rect_Tag {
-	gint16 left __attribute__((packed));
-	gint16 top __attribute__((packed));
-	gint16 right __attribute__((packed));
-	gint16 bottom __attribute__((packed));
-} Rect;
-/* AVI Stream Header */
-typedef struct AviStreamHeader_Tag {
-	guint32 type __attribute__((packed));
-	guint32 handler __attribute__((packed));
-	guint32 flags __attribute__((packed));
-	guint16 priority __attribute__((packed));
-	guint16 language __attribute__((packed));
-	guint32 initial_frames __attribute__((packed));
-	guint32 scale __attribute__((packed));
-	guint32 rate __attribute__((packed));
-	guint32 start __attribute__((packed));
-	guint32 length __attribute__((packed));
-	guint32 suggested_buffer_size __attribute__((packed));
-	guint32 quality __attribute__((packed));
-	guint32 sample_size __attribute__((packed));
-	Rect frame __attribute__((packed));
-} AviStreamHeader;
-/* AVI Index Entry */
-typedef struct AviIndexEntry_Tag {
-	guint32 ckid __attribute__((packed));
-	guint32 flags __attribute__((packed));
-	guint32 offset __attribute__((packed));
-	guint32 length __attribute__((packed));
-} AviIndexEntry;
-
-
-#else
-
-
-typedef struct { guint8 dummy[AMH_SIZE]; } AviMainHeader;
-typedef struct { guint8 dummy[RECT_SIZE]; } Rect;
-typedef struct { guint8 dummy[ASH_SIZE]; } AviStreamHeader;
-typedef struct { guint8 dummy[AIE_SIZE]; } AviIndexEntry;
-
-
+#ifdef __cplusplus
+extern "C" {
 #endif
 
 
-/******************************************************************************
-*                                                                             *
-* ¤Υޥ                                                                *
-*                                                                             *
-******************************************************************************/
-#define amh_get_micro_sec_per_frame(amh) GUINT32_FROM_LE(*(guint32 *)(amh))
-#define amh_get_max_bytes_per_sec(amh)										\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+4))
-#define amh_get_padding_granularity(amh)									\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+8))
-#define amh_get_flags(amh) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+12))
-#define amh_get_total_frames(amh)											\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+16))
-#define amh_get_initial_frames(amh)											\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+20))
-#define amh_get_streams(amh) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+24))
-#define amh_get_suggested_buffer_size(amh)									\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+28))
-#define amh_get_width(amh) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+32))
-#define amh_get_height(amh) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(amh)+36))
-#define amh_set_micro_sec_per_frame(amh,micro_sec_per_frame)				\
-						(*(guint32 *)(amh)=GUINT32_TO_LE(micro_sec_per_frame))
-#define amh_set_max_bytes_per_sec(amh,max_bytes_per_sec)					\
-			(*(guint32 *)((guint8 *)(amh)+4)=GUINT32_TO_LE(max_bytes_per_sec))
-#define amh_set_padding_granularity(amh,padding_granularity)				\
-		(*(guint32 *)((guint8 *)(amh)+8)=GUINT32_TO_LE(padding_granularity))
-#define amh_set_flags(amh,flags)											\
-						(*(guint32 *)((guint8 *)(amh)+12)=GUINT32_TO_LE(flags))
-#define amh_set_total_frames(amh,total_frames)								\
-				(*(guint32 *)((guint8 *)(amh)+16)=GUINT32_TO_LE(total_frames))
-#define amh_set_initial_frames(amh,initial_frames)							\
-			(*(guint32 *)((guint8 *)(amh)+20)=GUINT32_TO_LE(initial_frames))
-#define amh_set_streams(amh,streams)										\
-					(*(guint32 *)((guint8 *)(amh)+24)=GUINT32_TO_LE(streams))
-#define amh_set_suggested_buffer_size(amh,suggested_buffer_size)			\
-		(*(guint32 *)((guint8 *)(amh)+28)=GUINT32_TO_LE(suggested_buffer_size))
-#define amh_set_width(amh,width)											\
-						(*(guint32 *)((guint8 *)(amh)+32)=GUINT32_TO_LE(width))
-#define amh_set_height(amh,height)											\
-					(*(guint32 *)((guint8 *)(amh)+36)=GUINT32_TO_LE(height))
-
-
-#define ash_get_type(ash) GUINT32_FROM_LE(*(guint32 *)(ash))
-#define ash_get_handler(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+4))
-#define ash_get_flags(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+8))
-#define ash_get_priority(ash) GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+12))
-#define ash_get_language(ash) GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+14))
-#define ash_get_initial_frames(ash)											\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+16))
-#define ash_get_scale(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+20))
-#define ash_get_rate(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+24))
-#define ash_get_start(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+28))
-#define ash_get_length(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+32))
-#define ash_get_suggested_buffer_size(ash)									\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+36))
-#define ash_get_quality(ash) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+40))
-#define ash_get_sample_size(ash)											\
-							GUINT32_FROM_LE(*(guint32 *)((guint8 *)(ash)+44))
-#define ash_get_frame_left(ash)												\
-							GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+48))
-#define ash_get_frame_top(ash)												\
-							GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+50))
-#define ash_get_frame_right(ash)											\
-							GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+52))
-#define ash_get_frame_bottom(ash)											\
-							GUINT16_FROM_LE(*(guint16 *)((guint8 *)(ash)+54))
-#define ash_set_type(ash,type) (*(guint32 *)(ash)=GUINT32_TO_LE(type))
-#define ash_set_handler(ash,handler)										\
-					(*(guint32 *)((guint8 *)(ash)+4)=GUINT32_TO_LE(handler))
-#define ash_set_flags(ash,flags)											\
-						(*(guint32 *)((guint8 *)(ash)+8)=GUINT32_TO_LE(flags))
-#define ash_set_priority(ash,priority)										\
-					(*(guint16 *)((guint8 *)(ash)+12)=GUINT16_TO_LE(priority))
-#define ash_set_language(ash,language)										\
-					(*(guint16 *)((guint8 *)(ash)+14)=GUINT16_TO_LE(language))
-#define ash_set_initial_frames(ash,initial_frames)							\
-			(*(guint32 *)((guint8 *)(ash)+16)=GUINT32_TO_LE(initial_frames))
-#define ash_set_scale(ash,scale)											\
-						(*(guint32 *)((guint8 *)(ash)+20)=GUINT32_TO_LE(scale))
-#define ash_set_rate(ash,rate)												\
-						(*(guint32 *)((guint8 *)(ash)+24)=GUINT32_TO_LE(rate))
-#define ash_set_start(ash,start)											\
-						(*(guint32 *)((guint8 *)(ash)+28)=GUINT32_TO_LE(start))
-#define ash_set_length(ash,length)											\
-					(*(guint32 *)((guint8 *)(ash)+32)=GUINT32_TO_LE(length))
-#define ash_set_suggested_buffer_size(ash,suggested_buffer_size)			\
-		(*(guint32 *)((guint8 *)(ash)+36)=GUINT32_TO_LE(suggested_buffer_size))
-#define ash_set_quality(ash,quality)										\
-					(*(guint32 *)((guint8 *)(ash)+40)=GUINT32_TO_LE(quality))
-#define ash_set_sample_size(ash,sample_size)								\
-				(*(guint32 *)((guint8 *)(ash)+44)=GUINT32_TO_LE(sample_size))
-#define ash_set_frame_left(ash,frame_left)									\
-				(*(guint16 *)((guint8 *)(ash)+48)=GUINT16_TO_LE(frame_left))
-#define ash_set_frame_top(ash,frame_top)									\
-				(*(guint16 *)((guint8 *)(ash)+50)=GUINT16_TO_LE(frame_top))
-#define ash_set_frame_right(ash,frame_right)								\
-				(*(guint16 *)((guint8 *)(ash)+52)=GUINT16_TO_LE(frame_right))
-#define ash_set_frame_bottom(ash,frame_bottom)								\
-				(*(guint16 *)((guint8 *)(ash)+54)=GUINT16_TO_LE(frame_bottom))
-
-
-#define aie_get_ckid(aie) GUINT32_FROM_LE(*(guint32 *)(aie))
-#define aie_get_flags(aie) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(aie)+4))
-#define aie_get_offset(aie) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(aie)+8))
-#define aie_get_length(aie) GUINT32_FROM_LE(*(guint32 *)((guint8 *)(aie)+12))
-
-#define aie_set_type(aie,ckid) (*(guint32 *)(aie)=GUINT32_TO_LE(ckid))
-#define aie_set_flags(aie,flags)											\
-						(*(guint32 *)((guint8 *)(aie)+4)=GUINT32_TO_LE(flags))
-#define aie_set_offset(aie,offset)											\
-						(*(guint32 *)((guint8 *)(aie)+8)=GUINT32_TO_LE(offset))
-#define aie_set_length(aie,length)											\
-					(*(guint32 *)((guint8 *)(aie)+12)=GUINT32_TO_LE(length))
-
-
 /******************************************************************************
 *                                                                             *
 * Խ¤                                                                  *
 *                                                                             *
 ******************************************************************************/
+#define AVI_TYPE_UNKNOW		(-1)
+#define AVI_TYPE_AVI		0
+#define AVI_TYPE_BITMAP		1
+#define AVI_TYPE_WAVE		2
+#define AVI_TYPE_SCENARIO	3
+
+
+#define AVI_EDIT_CACHE		20
+
+
 typedef struct AviIndex_Tag {
 	guint64 offset;
 	guint32 size;
 	guint32 flags;
 } AviIndex;
 typedef struct AviFile_Tag {
-	int fd;						/* եǥ꥿ */
-	gpointer *data;				/*  */
-	gchar *name;				/* ե̾ */
-	gint start,length;			/* ͭϰ */
-	gint entries;				/* ǥåο */
-	gint param;					/* ȥ꡼ֹ */
-	AviIndex *index;			/* ǥå */
+	int fd;							/* եǥ꥿ */
+	gpointer *data;					/*  */
+	gchar *name;					/* ե̾ */
+	gint start,length;				/* ͭϰ */
+	gint entries;					/* ǥåο */
+	gint param;						/* ȥ꡼ֹ */
+	AviIndex *index;				/* ǥå */
 	/* ꥹ */
 	struct AviFile_Tag *prev,*next;
 	/* ӥǥͭ */
-	BitmapInfoHeader *bmih;		/* ꥸʥΥեޥå */
+	guint32 handler;
+	BitmapInfoHeader *bmih;			/* ꥸʥΥեޥå */
 	/* ǥͭ */
-	WaveFormatEx *wfx;			/* ꥸʥΥեޥå */
+	WaveFormatEx *wfx;				/* ꥸʥΥեޥå */
 } AviFile;
+typedef struct AviBuffer_Tag {
+	gpointer data;					/* å */
+	/* ӥǥͭ */
+	gboolean raw;					/* TRUE:32ӥåRAW,FALSE:ӥåȥޥå */
+	gint pos;						/*  */
+	gint width,height;				/* (RAWͭ) */
+} AviBuffer;
 typedef struct AviEdit_Tag {
 	gint offset;
 	AviFile *file;
-	guint32 type;				/* streamtypeAUDIO/streamtypeVIDEO */
-	gint length;				/* ץ */
+	guint32 type;					/* streamtypeAUDIO/streamtypeVIDEO */
+	gint length;					/* ץ */
 	guint32 rate;
-	guint32 scale;				/* ratescale=1ô֤Υե졼 */
-	gpointer buf;				/* ŸΥǡ */
+	guint32 scale;					/* ratescale=1ô֤Υե졼 */
+	guint16 priority;				/* ͥ */
+	guint16 language;				/*  */
+	gchar *name;					/* ̾ */
+	AviBuffer buf[AVI_EDIT_CACHE];	/* å */
 	/* ӥǥͭ */
-	BitmapInfoHeader *bmih;		/* ŸΥեޥå */
-	gint pos;					/* Ǹ˼ե졼 */
+	BitmapInfoHeader *bmih;			/* ŸΥեޥå */
 	/* ǥͭ */
-	WaveFormatEx *wfx;			/* ŸΥեޥå */
+	WaveFormatEx *wfx;				/* ŸΥեޥå */
 } AviEdit;
+typedef struct AviFrame_Tag {
+	guint32 handler;				/* 4ʸ */
+	gint pos;						/* Ǹ˼ե졼 */
+	AviEdit *avi_edit;
+	IcmObject *icm_object;			/* ICMŸ */
+} AviFrame;
+typedef struct AviOptions_Tag {
+	guint32 type;					/* streamtypeAUDIO/streamtypeVIDEO */
+	/* ӥǥͭ */
+	guint32 handler;				/* 4ʸ */
+	gint key_frame;					/* (0:٤ƥե졼) */
+	gint quality;					/* ʼ010000,-1:ǥե */
+	gsize data_rate;				/* 1ô֤Υ(0:̵) */
+	gboolean recompress;			/* TRUE:ư̤,FALSE:ư̤ʤ */
+	gpointer param;					/* ץǡ */
+	gsize param_size;				/* ץǡΥ */
+	BitmapInfoHeader *bmih;			/* ̸Υեޥå */
+	gsize bmih_size;				/* ̸ΥեޥåȤΥ */
+	/* ǥͭ */
+	WaveFormatEx *wfx;				/* ̸Υեޥå */
+} AviOptions;
+typedef struct AviSave_Tag {
+	gint type;						/* ¸եμ */
+	/* AVIͭ */
+	AviOptions *avi_opt;			/* ץ */
+	/* ӥåȥޥå׸ͭ */
+	gint stream;					/* оݥȥ꡼ */
+	gint start;						/* ǽΥե졼 */
+	gint end;						/* ǸΥե졼 */
+} AviSave;
 
 
 /******************************************************************************
@@ -285,6 +132,36 @@ typedef struct AviEdit_Tag {
 * եޥåȥޥ                                                          *
 *                                                                             *
 ******************************************************************************/
+/*	AVIRate
+	avi_edit,AVIԽϥɥ
+	     RET,Rate															*/
+#define avi_get_rate(avi_edit) (((AviEdit *)(avi_edit))->rate)
+
+
+/*	AVIScale
+	avi_edit,AVIԽϥɥ
+	     RET,Scale															*/
+#define avi_get_scale(avi_edit) (((AviEdit *)(avi_edit))->scale)
+
+
+/*	AVIͥ٤
+	avi_edit,AVIԽϥɥ
+	     RET,ͥ															*/
+#define avi_get_priority(avi_edit) (((AviEdit *)(avi_edit))->priority)
+
+
+/*	AVIθ
+	avi_edit,AVIԽϥɥ
+	     RET,															*/
+#define avi_get_language(avi_edit) (((AviEdit *)(avi_edit))->language)
+
+
+/*	AVI̾
+	avi_edit,AVIԽϥɥ
+	     RET,̾															*/
+#define avi_get_name(avi_edit) (((AviEdit *)(avi_edit))->name)
+
+
 /*	AVI
 	avi_edit,AVIԽϥɥ
 	     RET,,0:顼													*/
@@ -306,6 +183,13 @@ typedef struct AviEdit_Tag {
 				((avi_edit)->bmih!=NULL?bmih_get_bit_count((avi_edit)->bmih):0)
 
 
+/*	AVIΰ̷
+	avi_edit,AVIԽϥɥ
+	     RET,̷,-1:顼												*/
+#define avi_get_compression(avi_edit)										\
+	((avi_edit)->bmih!=NULL?bmih_get_compression((avi_edit)->bmih):(guint32)-1)
+
+
 /*	AVIX٤
 	avi_edit,AVIԽϥɥ
 	     RET,X,-1:顼												*/
@@ -341,18 +225,56 @@ typedef struct AviEdit_Tag {
 			((avi_edit)->wfx!=NULL?wfx_get_bits_per_sample((avi_edit)->wfx):0)
 
 
+/*	AVIRateꤹ
+	avi_edit,AVIԽϥɥ
+	   value,Rate															*/
+#define avi_set_rate(avi_edit,value) (((AviEdit *)(avi_edit))->rate=(value))
+
+
+/*	AVIScaleꤹ
+	avi_edit,AVIԽϥɥ
+	   value,Scale															*/
+#define avi_set_scale(avi_edit,value) (((AviEdit *)(avi_edit))->scale=(value))
+
+
+/*	AVIͥ٤ꤹ
+	avi_edit,AVIԽϥɥ
+	   value,ͥ															*/
+#define avi_set_priority(avi_edit,value)									\
+									(((AviEdit *)(avi_edit))->priority=(value))
+
+
+/*	AVIθꤹ
+	avi_edit,AVIԽϥɥ
+	   value,															*/
+#define avi_set_language(avi_edit,value)									\
+									(((AviEdit *)(avi_edit))->language=(value))
+
+
+/*	AVI̾ꤹ
+	avi_edit,AVIԽϥɥ
+	   value,̾															*/
+#define avi_set_name(avi_edit,value)										\
+	(																		\
+		g_free(((AviEdit *)(avi_edit))->name),								\
+		((AviEdit *)(avi_edit))->name=value!=NULL?g_strdup(value):NULL		\
+	)
+
+
 /*	AVIꤹ
 	avi_edit,AVIԽϥɥ
 	   width,																*/
 #define avi_set_width(avi_edit,width)										\
-			((avi_edit)->bmih!=NULL?bmih_set_width((avi_edit)->bmih,width):0)
+	((avi_edit)->bmih!=NULL?(bmih_set_width((avi_edit)->bmih,width),		\
+	bmih_set_size_image((avi_edit)->bmih,bm_image_bytes((avi_edit)->bmih))):0)
 
 
 /*	AVIι⤵ꤹ
 	avi_edit,AVIԽϥɥ
 	  height,⤵															*/
 #define avi_set_height(avi_edit,height)										\
-			((avi_edit)->bmih!=NULL?bmih_set_height((avi_edit)->bmih,height):0)
+	((avi_edit)->bmih!=NULL?(bmih_set_height((avi_edit)->bmih,height),		\
+	bmih_set_size_image((avi_edit)->bmih,bm_image_bytes((avi_edit)->bmih))):0)
 
 
 /*	AVIοꤹ
@@ -427,24 +349,13 @@ typedef struct AviEdit_Tag {
 #define avi_type(avi_edit) (((AviEdit *)(avi_edit))->type)
 
 
-/*	AVIRate
-	avi_edit,AVIԽϥɥ
-	     RET,Rate															*/
-#define avi_rate(avi_edit) (((AviEdit *)(avi_edit))->rate)
-
-
-/*	AVIScale
-	avi_edit,AVIԽϥɥ
-	     RET,Scale															*/
-#define avi_scale(avi_edit) (((AviEdit *)(avi_edit))->scale)
-
-
 /*	AVIΥץֹ椫֤
 	avi_edit,AVIԽϥɥ
 	  sample,ץֹ
 	     RET,(ߥ)													*/
 #define avi_sample_to_time(avi_edit,sample)									\
-		(gint)((guint64)(sample)*avi_scale(avi_edit)*1000/avi_rate(avi_edit))
+					(gint)((guint64)(sample)*avi_get_scale(avi_edit)*1000	\
+													/avi_get_rate(avi_edit))
 
 
 /*	AVIλ֤饵ץֹ
@@ -452,8 +363,8 @@ typedef struct AviEdit_Tag {
 	    time,(ߥ)
 	     RET,ץֹ													*/
 #define avi_time_to_sample(avi_edit,time)									\
-								(gint)((guint64)(time)*avi_rate(avi_edit)	\
-										/((guint64)avi_scale(avi_edit)*1000))
+							(gint)((guint64)(time)*avi_get_rate(avi_edit)	\
+									/((guint64)avi_get_scale(avi_edit)*1000))
 
 
 /*	AVIĹ
@@ -469,28 +380,17 @@ typedef struct AviEdit_Tag {
 							avi_sample_to_time(avi_edit,avi_length(avi_edit))
 
 
-/******************************************************************************
-*                                                                             *
-* Խޥ                                                                  *
-*                                                                             *
-******************************************************************************/
-/*	AVIʣ
-	avi_edit,AVIԽϥɥ
-	     RET,AVIԽϥɥ												*/
-#define avi_clone(avi_edit) avi_copy(avi_edit,0,avi_length(avi_edit))
-
-
 /******************************************************************************
 *                                                                             *
 * ؿ                                                                  *
 *                                                                             *
 ******************************************************************************/
 /*	AVI															*/
-void avi_file_init(void);
+gboolean avi_init(void);
 
 
 /*	AVIλ															*/
-void avi_file_exit(void);
+gboolean avi_exit(void);
 
 
 /******************************************************************************
@@ -559,13 +459,36 @@ gboolean avi_read_format(AviEdit *avi_edit,gpointer format);
 * ե졼ؿ(ߤ)                                                        *
 *                                                                             *
 ******************************************************************************/
+/*	DIBŸ줿ե졼򳫤
+	avi_edit,AVIԽϥɥ
+	     RET,AVIե졼๽¤,NULL:顼									*/
+AviFrame *avi_get_frame_open(AviEdit *avi_edit);
+
+
+/*	DIBŸ줿ե졼Ĥ
+	avi_frame,AVIե졼๽¤
+	      RET,TRUE:ｪλ,FALSE:顼									*/
+gboolean avi_get_frame_close(AviFrame *avi_frame);
+
+
 /*	DIBŸ줿ե졼
-	 avi_edit,AVIԽϥɥ
+	avi_frame,AVIե졼๽¤
 	      pos,ե졼(ץֹ)
 	    width,
 	   height,⤵
+	bit_count,
 	      RET,ѥåDIBؤΥݥ,NULL:顼								*/
-BitmapInfoHeader *avi_get_frame(AviEdit *avi_edit,const gint pos,
+BitmapInfoHeader *avi_get_frame(AviFrame *avi_frame,const gint pos,
+					const gint width,const gint height,const gint bit_count);
+
+
+/*	RAWŸ줿ե졼
+	avi_frame,AVIե졼๽¤
+	      pos,ե졼(ץֹ)
+	    width,
+	   height,⤵
+	      RET,RAWؤΥݥ,NULL:顼								*/
+guchar *avi_get_frame32(AviFrame *avi_frame,const gint pos,
 										const gint width,const gint height);
 
 
@@ -605,6 +528,12 @@ AviEdit *avi_copy(AviEdit *avi_edit,const gint start,const gint samples);
 gboolean avi_paste(AviEdit *avi_dst,const gint pos,AviEdit *avi_src);
 
 
+/*	AVIʣ
+	avi_edit,AVIԽϥɥ
+	     RET,AVIԽϥɥ												*/
+#define avi_clone(avi_edit) avi_copy(avi_edit,0,avi_length(avi_edit))
+
+
 /******************************************************************************
 *                                                                             *
 * ʥꥪ֥ȴؿ(פ)                                            *
@@ -628,6 +557,36 @@ AviEdit **avi_from_scenario(const gchar *scenario);
 * ¸ؿ()                                                            *
 *                                                                             *
 ******************************************************************************/
+/*	¸ѤΥץ
+	avi_save,ץ
+	     RET,TRUE:ｪλ,FALSE:顼										*/
+gboolean avi_save_free(AviSave *avi_save);
+
+
+/*	٤¸뤫å
+	avi_edit,AVIԽϥɥ
+	avi_save,ץ
+	     RET,TRUE:٤¸,FALSE:٤¸ʤ					*/
+#define avi_save_is_all(avi_edit,avi_save) ((AviEdit **)(avi_edit)!=NULL	\
+		&& ((AviEdit **)(avi_edit))[0]!=NULL && (AviSave *)(avi_save)!=NULL	\
+		&& (((AviSave *)(avi_save))->type==AVI_TYPE_AVI						\
+			|| ((AviSave *)(avi_save))->type==AVI_TYPE_SCENARIO				\
+			|| (((AviEdit **)(avi_edit))[1]==NULL							\
+							&& ((AviSave *)(avi_save))->type==AVI_TYPE_WAVE)))
+
+
+/*	ե¸
+	 avi_edit,AVIԽϥɥؤΥݥ
+	     file,ե̾
+	  avi_opt,ץ
+	     func,Хåؿ
+	user_data,ǡ
+	      RET,TRUE:ｪλ,FALSE:顼									*/
+gboolean avi_save_avi(AviEdit *avi_edit[],const gchar *file,
+						AviOptions *avi_opt,
+						gboolean (*func)(gint,gpointer),gpointer user_data);
+
+
 /*	ӥåȥޥåץե¸
 	 avi_edit,AVIԽϥɥ
 	     file,ե̾
@@ -654,11 +613,35 @@ gboolean avi_save_wave(AviEdit *avi_edit,const gchar *file,
 /*	ե¸
 	 avi_edit,AVIԽϥɥؤΥݥ
 	     file,ե̾
+	 avi_save,ץ
 	     func,Хåؿ
 	user_data,ǡ
 	      RET,TRUE:ｪλ,FALSE:顼									*/
-gboolean avi_save(AviEdit *avi_edit[],const gchar *file,
-						gboolean (*func)(gint,gpointer),gpointer user_data);
+gboolean avi_save_with_options(AviEdit *avi_edit[],const gchar *file,
+		AviSave *avi_save,gboolean (*func)(gint,gpointer),gpointer user_data);
 
 
+/******************************************************************************
+*                                                                             *
+* ؿ()                                                      *
+*                                                                             *
+******************************************************************************/
+/*	¸ѤΥץ
+	avi_edit,AVIԽϥɥؤΥݥ
+	     RET,ץ,NULL:󥻥:顼								*/
+AviSave *avi_save_dialog(AviEdit *avi_edit[]);
+
+
+/*	¸ѤΥץ
+	avi_edit,AVIԽϥɥؤΥݥ
+	    file,򤷤ե
+	    path,ǥեȤΥǥ쥯ȥ
+	     RET,ץ,NULL:顼											*/
+AviSave *avi_save_dialog_with_file(AviEdit *avi_edit[],
+											gchar **file,const gchar *path);
+
+
+#ifdef __cplusplus
+}
+#endif
 #endif
