@@ -47,7 +47,7 @@ gint avi_file_type(const gchar *file)
 		return AVI_TYPE_UNKNOW;
 	if (
 #ifdef HAVE_FLOCK
-		flock(fd,LOCK_SH)==-1 ||
+		flock (fd, LOCK_SH) != 0 ||
 #endif
 						read(fd,buffer,sizeof(guint8)*12)!=sizeof(guint8)*12) {
 		close(fd);
@@ -92,7 +92,7 @@ guint32 avi_main_streams(const gchar *file)
 	/* إåʬɤ߹ */
 	if (
 #ifdef HAVE_FLOCK
-		flock(fd,LOCK_SH)==-1 ||
+		flock (fd, LOCK_SH) != 0 ||
 #endif
 						read(fd,buffer,sizeof(guint8)*12)!=sizeof(guint8)*12) {
 		close(fd);
@@ -151,19 +151,18 @@ guint32 avi_main_streams(const gchar *file)
 	param,ȥ꡼ֹ
 	  RET,AVIԽϥɥ,NULL:顼										*/
 AviEdit *
-avi_open(const gchar *file,const gint param)
+avi_open (const gchar *file, const gint param)
 {
 	AviEdit *avi_edit;
 	IcmObject *icm_object;
 
-	/*  */
-	avi_edit = g_malloc (sizeof (AviEdit));
-	avi_edit->offset = 0;
-	/* å */
-	g_memset (avi_edit->buf, 0, AVI_EDIT_CACHE * sizeof (AviBuffer));
-
-	if (file == NULL || (avi_edit->file = avi_open_file(file, param, 
-				&avi_edit->type, &avi_edit->rate, &avi_edit->scale)) == NULL)
+	if (file == NULL)
+		return NULL;
+	avi_edit = g_malloc0 (sizeof (AviEdit));	/*  */
+	avi_edit->file = avi_open_file(file, param,
+					&avi_edit->type, &avi_edit->rate, &avi_edit->scale,
+					&avi_edit->priority, &avi_edit->language, &avi_edit->name);
+	if (avi_edit->file == NULL)
 	  {
 		g_free (avi_edit);
 		return NULL;
@@ -211,8 +210,6 @@ avi_open(const gchar *file,const gint param)
 			avi_edit->bmih = g_realloc (avi_edit->bmih,
 											bm_header_bytes (avi_edit->bmih));
 		  }
-		/* ǥͭ */
-		avi_edit->wfx = NULL;			/* ŸΥեޥå */
 	  }
 	else
 	  {
@@ -228,8 +225,6 @@ avi_open(const gchar *file,const gint param)
 			g_free (avi_edit);
 			return NULL;
 		  }
-		/* ӥǥͭ */
-		avi_edit->bmih = NULL;			/* ŸΥեޥå */
 	  }
 	/*  */
 	avi_edit->length = avi_edit->file->length;
@@ -252,6 +247,7 @@ gboolean avi_release(AviEdit *avi_edit)
 		avi_release_file(avi_edit->file);
 		avi_edit->file=avi_file;
 	}
+	g_free (avi_edit->name);
 	g_free(avi_edit->bmih);
 	for (i=0;i<AVI_EDIT_CACHE;i++)
 		g_free(avi_edit->buf[i].data);
