@@ -98,10 +98,12 @@ typedef struct Chunk_Tag {
 #define streamtypeAUDIO			make4cc('a','u','d','s')
 #define comptypeDIB				make4cc('D','I','B',' ')
 #define formtypeAVI				make4cc('A','V','I',' ')
+#define formtypeAVIX			make4cc('A','V','I','X')
 #define formtypeWAVE			make4cc('W','A','V','E')
 #define listtypeAVIHEADER		make4cc('h','d','r','l')
 #define listtypeSTREAMHEADER	make4cc('s','t','r','l')
 #define listtypeAVIMOVIE		make4cc('m','o','v','i')
+#define listtypeOPENDML			make4cc('o','d','m','l')
 #define ckidAVIMAINHDR			make4cc('a','v','i','h')
 #define ckidSTREAMHEADER		make4cc('s','t','r','h')
 #define ckidSTREAMFORMAT		make4cc('s','t','r','f')
@@ -111,6 +113,8 @@ typedef struct Chunk_Tag {
 #define ckidAVIPADDING			make4cc('J','U','N','K')
 #define ckidWAVEFORMAT			make4cc('f','m','t',' ')
 #define ckidWAVEDATA			make4cc('d','a','t','a')
+#define ckidAVISUPERINDEX		make4cc('i','n','d','x')
+#define ckidOPENDML				make4cc('d','m','l','h')
 
 
 /*	եǥ꥿󥯹¤Τ
