@@ -1,6 +1,6 @@
 /*
     avicore
-    copyright (c) 2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 2002-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,12 @@
 #include <gdk/gdkkeysyms.h>
 #include "misc/misc.h"
 #include "misc/profile.h"
+#if defined(W32CODECDIR) || defined(USE_CODEC)
+# include "codec_c.h"
+#endif /* W32CODECDIR || USE_CODEC */
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>  
+#endif /* HAVE_UNISTD_H */
 
 
 /******************************************************************************
@@ -48,7 +54,7 @@
 }*/
 
 
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 /*	ʸ4ʸ
 	text,ʸ
 	 RET,4ʸ														*/
@@ -82,7 +88,7 @@ static guint32 icm_string_to_handler(const gchar *text)
 	}
 	return handler;
 }
-#endif /* W32CODECDIR */
+#endif /* defined(W32CODECDIR) || defined(USE_CODEC) */
 
 
 /******************************************************************************
@@ -93,36 +99,67 @@ static guint32 icm_string_to_handler(const gchar *text)
 static GHashTable *ghash_handler=NULL,*ghash_path=NULL;
 
 
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 static void icm_init_callback(gpointer key,gpointer value,gpointer ghash)
 {
 	gchar *path;
 	guint32 handler;
 	IcmDriver *icm_driver;
 
-	if (g_hash_table_lookup_extended((GHashTable*)ghash,key,
-									(gpointer *)&handler,(gpointer *)&path)) {
-		if ((icm_driver=g_hash_table_lookup(ghash_path,path))==NULL) {
-			/*  */
-			if ((icm_driver=g_malloc0(sizeof(IcmDriver)))!=NULL
-					&& (icm_driver->pe=peimage_create())!=NULL
-					&& peimage_load(icm_driver->pe,path)
-					&& (icm_driver->DriverProc=peimage_resolve(icm_driver->pe,
-														"DriverProc"))!=NULL) {
-				icm_driver->DriverProc(0,icm_driver->pe,DRV_LOAD,0,0);
-				icm_driver->DriverProc(0,icm_driver->pe,DRV_ENABLE,0,0);
-				g_hash_table_insert(ghash_path,path,icm_driver);
-			} else {
-				if (icm_driver->pe!=NULL)
-					peimage_destroy(icm_driver->pe);
-				g_free(icm_driver);
-				g_free(path);
-				icm_driver=NULL;
-			}
-		}
-		if (icm_driver!=NULL)	/* Ͽ */
-			g_hash_table_insert(ghash_handler,(gpointer)handler,icm_driver);
+	if (g_hash_table_lookup_extended((GHashTable*)ghash, key,
+	    (gpointer *)&handler, (gpointer *)&path) == 0)
+		return;
+	if ((icm_driver = g_hash_table_lookup(ghash_path, path)) != NULL) {
+		g_free(path);
+		return;
 	}
+	/*  */
+	if ((icm_driver = g_malloc0(sizeof(IcmDriver))) == NULL) {
+		g_free(path);
+		return;
+	}
+#if defined(W32CODECDIR)
+	if (isPE(path)) {
+		if (((icm_driver->pe = peimage_create()) != NULL)
+		    && peimage_load(icm_driver->pe, path)
+		    && (icm_driver->DriverProc = peimage_resolve(icm_driver->pe,
+		    "DriverProc"))!=NULL) {
+			icm_driver->DriverProc(0,icm_driver->pe, DRV_LOAD, 0, 0);
+			icm_driver->DriverProc(0,icm_driver->pe, DRV_ENABLE, 0, 0);
+			g_hash_table_insert(ghash_path, path, icm_driver);
+			g_hash_table_insert(ghash_handler, (gpointer)handler, icm_driver);
+			icm_driver->pe->codec_type = ICM_ATTR_WIN32;
+			return;
+		}
+	} else
+#endif /* defined(W32CODECDIR) */
+#if defined(USE_CODEC)
+	if ((icm_driver->pe = codec_load(path)) != NULL) {
+		if ((icm_driver->DriverProc = codec_get_driverproc(icm_driver->pe)) == NULL){
+      		g_message("icm.c : %s : load failed.codec_get_driverproc", __FUNCTION__);
+			return;
+		}
+      		g_message("icm.c : %s : load success.", __FUNCTION__);
+		icm_driver->DriverProc(0, icm_driver->pe, DRV_LOAD, 0, 0);
+		icm_driver->DriverProc(0, icm_driver->pe, DRV_ENABLE, 0, 0);
+		g_hash_table_insert(ghash_path, path, icm_driver);
+		g_hash_table_insert(ghash_handler, (gpointer)handler, icm_driver);
+		icm_driver->pe->codec_type = ICM_ATTR_NATIVE;
+		return;
+	} else
+#endif /*defined(USE_CODEC) */
+	{
+      		g_message("icm.c : %s : load failed.", __FUNCTION__);
+		/*dummy*/
+	}
+	/* falldown,if fail */
+	if(icm_driver!=NULL){
+		if(icm_driver->pe!=NULL)
+			peimage_destroy(icm_driver->pe);
+		g_free(icm_driver);
+		icm_driver=NULL;
+	}
+	g_free(path);
 }
 #endif
 
@@ -131,8 +168,8 @@ static void icm_init_callback(gpointer key,gpointer value,gpointer ghash)
 	RET,TRUE:ｪλ,FALSE:顼											*/
 gboolean icm_init(void)
 {
-#ifdef W32CODECDIR
-	gchar *key,*value,*path;
+#if defined(W32CODECDIR) || defined(USE_CODEC)
+	gchar *key,*value,*path,*fpath;
 	gint i,length;
 	guint32 handler;
 	GList *glist;
@@ -150,23 +187,41 @@ gboolean icm_init(void)
 				key=g_list_nth_data(glist,i);
 				if ((handler=icm_string_to_handler(key))!=0
 					&& (value=profile_get_string(profile,"vidc",key))!=NULL) {
-					path=g_hash_table_lookup(ghash,(gconstpointer)handler);
-					g_free(path);
+
 					if (value[0]=='\0' || value[g_strlen(value)-1]=='/') {
-						if (path!=NULL)
-							g_hash_table_remove(ghash,(gconstpointer)handler);
+						if ((path = g_hash_table_lookup(ghash, (gconstpointer)handler)))
+							g_free(path);
 						g_free(value);
 					} else {
-						if (g_path_is_absolute(value)) {
+						if (g_path_is_absolute(value))
 							path=value;
-						} else {
-							path=g_strconcat(W32CODECDIR,"/",value,NULL);
+						else {
+				//XXX: TODO:NATIVECODECDIRȤ褦ѹ
+#if defined(USE_CODEC)
+							path = g_strconcat("/home/codec", "/", value, NULL);
+							if (access(path, F_OK) != 0) {
+								g_free(path);
+#endif /* defined(USE_CODEC) */
+#if defined(W32CODECDIR)
+								path = g_strconcat(W32CODECDIR, "/", value, NULL);
+								if (access(path, F_OK) != 0) {
+									g_free(path);
+#endif /* defined(W32CODECDIR) */
+									g_free(value);
+									continue;
+#if defined(W32CODECDIR)
+								}
+#endif /* defined(W32CODECDIR) */
+#if defined(USE_CODEC)
+							}
+#endif /* defined(USE_CODEC) */
 							g_free(value);
 						}
-						value=path;
-						path=misc_get_full_path(path);
-						g_free(value);
-						g_hash_table_insert(ghash,(gpointer)handler,path);
+						fpath = misc_get_full_path(path);
+						g_free(path);
+						if ((path = g_hash_table_lookup(ghash, (gconstpointer)handler)))
+							g_free(path);
+						g_hash_table_insert(ghash, (gpointer)handler, fpath);
 					}
 				}
 			}
@@ -190,7 +245,7 @@ static void icm_exit_callback(gpointer key,gpointer value,gpointer ghash)
 
 	if (g_hash_table_lookup_extended((GHashTable*)ghash,key,
 								(gpointer *)&path,(gpointer *)&icm_driver)) {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 		icm_driver->DriverProc(0,icm_driver->pe,DRV_FREE,0,0);
 		peimage_destroy(icm_driver->pe);
 #endif
@@ -246,7 +301,7 @@ GList *icm_get_handler_list(void)
 	    RET,IcmObject,NULL:顼											*/
 IcmObject *icm_open(const guint32 handler,const gint mode)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	IcmObject *icm_object;
 	ICOPEN icopen;
 
@@ -300,7 +355,7 @@ IcmObject *icm_open(const guint32 handler,const gint mode)
 gboolean icm_close(IcmObject *icm_object)
 {
 	if (icm_object!=NULL && --(icm_object->counter)<=0) {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 		icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 									icm_object->icm_driver->pe,DRV_CLOSE,0,0);
 #endif
@@ -319,7 +374,7 @@ gboolean icm_close(IcmObject *icm_object)
 	       RET,TRUE:ｪλ,FALSE:顼									*/
 gboolean icm_get_info(IcmObject *icm_object,IcmInfo *icm_info)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	ICINFO icinfo;
 
 	if (icm_object==NULL)
@@ -328,7 +383,12 @@ gboolean icm_get_info(IcmObject *icm_object,IcmInfo *icm_info)
 	icinfo.dwSize=sizeof(ICINFO);
 	icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 		icm_object->icm_driver->pe,ICM_GETINFO,(LONG)&icinfo,sizeof(ICINFO));
-	icm_info->name=misc_widestring_to_multistring(icinfo.szName);
+#if defined(USE_CODEC)
+	if (icm_object->icm_driver->pe->codec_type == ICM_ATTR_NATIVE)
+		icm_info->name = g_strdup((gchar *)icinfo.szName);
+	else
+#endif /* defined(USE_CODEC) */
+		icm_info->name=misc_widestring_to_multistring(icinfo.szName);
 	icm_info->handler=icinfo.fccHandler;
 	icm_info->flags=0;
 	if ((icinfo.dwFlags&VIDCF_QUALITY)!=0)
@@ -347,7 +407,7 @@ gboolean icm_get_info(IcmObject *icm_object,IcmInfo *icm_info)
 		icm_info->flags|=ICM_FLAG_FASTTEMPORALD;
 	if ((icinfo.dwFlags&VIDCF_QUALITYTIME)!=0)
 		icm_info->flags|=ICM_FLAG_QUALITYTIME;
-	icm_info->attribute=ICM_ATTR_WIN32;
+	icm_info->attribute = icm_object->icm_driver->pe->codec_type;
 	return TRUE;
 #else
 	return FALSE;
@@ -360,7 +420,7 @@ gboolean icm_get_info(IcmObject *icm_object,IcmInfo *icm_info)
 	       RET,														*/
 gsize icm_get_state_size(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL
 				&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 								icm_object->icm_driver->pe,ICM_GETSTATE,0,0);
@@ -377,7 +437,7 @@ gsize icm_get_state_size(IcmObject *icm_object)
 	       RET,TRUE:ͭ,FALSE:̵											*/
 gboolean icm_get_state(IcmObject *icm_object,gpointer param,gsize param_size)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && param!=NULL && param_size>0
 				&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 									icm_object->icm_driver->pe,ICM_GETSTATE,
@@ -395,7 +455,7 @@ gboolean icm_get_state(IcmObject *icm_object,gpointer param,gsize param_size)
 	       RET,TRUE:ͭ,FALSE:̵											*/
 gboolean icm_set_state(IcmObject *icm_object,gpointer param,gsize param_size)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object!=NULL && param!=NULL && param_size>0) {
 		icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 									icm_object->icm_driver->pe,ICM_SETSTATE,
@@ -417,7 +477,7 @@ gboolean icm_set_state(IcmObject *icm_object,gpointer param,gsize param_size)
 	       RET,TRUE:ͭ,FALSE:̵											*/
 gboolean icm_is_dialog_about(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && icm_object->icm_driver->DriverProc
 							(icm_object->dwDriverId,icm_object->icm_driver->pe,
 													ICM_ABOUT,-1,0)==ICERR_OK;
@@ -431,7 +491,7 @@ gboolean icm_is_dialog_about(IcmObject *icm_object)
 	icm_object,IcmObject													*/
 void icm_dialog_about(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object!=NULL)
 		icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 									icm_object->icm_driver->pe,ICM_ABOUT,0,0);
@@ -444,7 +504,7 @@ void icm_dialog_about(IcmObject *icm_object)
 	       RET,TRUE:ͭ,FALSE:̵											*/
 gboolean icm_is_dialog_configure(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && icm_object->icm_driver->DriverProc
 							(icm_object->dwDriverId,icm_object->icm_driver->pe,
 												ICM_CONFIGURE,-1,0)==ICERR_OK;
@@ -458,7 +518,7 @@ gboolean icm_is_dialog_configure(IcmObject *icm_object)
 	icm_object,IcmObject													*/
 void icm_dialog_configure(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object!=NULL)
 		icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 								icm_object->icm_driver->pe,ICM_CONFIGURE,0,0);
@@ -477,7 +537,7 @@ void icm_dialog_configure(IcmObject *icm_object)
 	       RET,TRUE:ǽ,FALSE:Բǽ										*/
 gboolean icm_compress_query(IcmObject *icm_object,const BitmapInfoHeader *bmih)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && bmih!=NULL
 				&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 									icm_object->icm_driver->pe,
@@ -493,7 +553,7 @@ gboolean icm_compress_query(IcmObject *icm_object,const BitmapInfoHeader *bmih)
 	       RET,ǥե(0:̤б)											*/
 gint icm_compress_get_default_key_frame(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	DWORD dwKeyFrame;
 
 	return icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
@@ -510,7 +570,7 @@ gint icm_compress_get_default_key_frame(IcmObject *icm_object)
 	       RET,ǥե(-1:̤б)										*/
 gint icm_compress_get_default_quality(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	DWORD dwQuality;
 
 	return icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
@@ -529,7 +589,7 @@ gint icm_compress_get_default_quality(IcmObject *icm_object)
 gsize icm_compress_get_format_size(IcmObject *icm_object,
 												const BitmapInfoHeader *bmih)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL || bmih==NULL)
 		return 0;
 	return icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
@@ -549,7 +609,7 @@ gsize icm_compress_get_format_size(IcmObject *icm_object,
 gboolean icm_compress_get_format(IcmObject *icm_object,
 					const BitmapInfoHeader *bmih_in,BitmapInfoHeader *bmih_out)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && bmih_in!=NULL && bmih_out!=NULL
 		&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 			icm_object->icm_driver->pe,
@@ -565,7 +625,7 @@ gboolean icm_compress_get_format(IcmObject *icm_object,
 	       RET,Хȿ,0:顼											*/
 gsize icm_compress_get_size(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL
 					&& icm_object->bmih_in!=NULL && icm_object->bmih_out!=NULL
 				?icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
@@ -594,7 +654,7 @@ gboolean icm_compress_frames_info(IcmObject *icm_object,
 				const gint key_frame,const gint quality,const gsize data_rate,
 										const guint32 rate,const guint32 scale)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	IcmInfo icm_info;
 	ICCOMPRESSFRAMES iccompressframes;
 
@@ -652,7 +712,7 @@ gboolean icm_compress_frames_info(IcmObject *icm_object,
 	       RET,TRUE:ｪλ,FALSE:顼									*/
 gboolean icm_compress_begin(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL || icm_object->bmih_in==NULL
 													|| icm_object->prev!=NULL)
 		return FALSE;
@@ -685,7 +745,7 @@ gboolean icm_compress_begin(IcmObject *icm_object)
 	       RET,TRUE:ｪλ,FALSE:顼									*/
 gboolean icm_compress_end(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL || icm_object->prev==NULL)
 		return FALSE;
 	g_free(icm_object->prev);
@@ -707,7 +767,7 @@ gboolean icm_compress_end(IcmObject *icm_object)
 gsize icm_compress(IcmObject *icm_object,
 							gboolean *key_frame,gpointer input,gpointer output)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	DWORD dwFlags,dwID;
 	ICCOMPRESS iccompress;
 
@@ -759,7 +819,7 @@ gsize icm_compress(IcmObject *icm_object,
 gboolean icm_decompress_query(IcmObject *icm_object,
 												const BitmapInfoHeader *bmih)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && bmih!=NULL
 				&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 								icm_object->icm_driver->pe,
@@ -777,7 +837,7 @@ gboolean icm_decompress_query(IcmObject *icm_object,
 gsize icm_decompress_get_format_size(IcmObject *icm_object,
 												const BitmapInfoHeader *bmih)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL || bmih==NULL)
 		return 0;
 	return icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
@@ -797,7 +857,7 @@ gsize icm_decompress_get_format_size(IcmObject *icm_object,
 gboolean icm_decompress_get_format(IcmObject *icm_object,
 					const BitmapInfoHeader *bmih_in,BitmapInfoHeader *bmih_out)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	return icm_object!=NULL && bmih_in!=NULL && bmih_out!=NULL
 		&& icm_object->icm_driver->DriverProc(icm_object->dwDriverId,
 			icm_object->icm_driver->pe,
@@ -815,7 +875,7 @@ gboolean icm_decompress_get_format(IcmObject *icm_object,
 gboolean icm_decompress_begin(IcmObject *icm_object,
 												const BitmapInfoHeader *bmih)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL || bmih==NULL
 					|| icm_object->bmih_in!=NULL || icm_object->bmih_out!=NULL)
 		return FALSE;
@@ -843,7 +903,7 @@ gboolean icm_decompress_begin(IcmObject *icm_object,
 	       RET,TRUE:ｪλ,FALSE:顼									*/
 gboolean icm_decompress_end(IcmObject *icm_object)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	if (icm_object==NULL
 				|| (icm_object->bmih_in==NULL && icm_object->bmih_out==NULL))
 		return FALSE;
@@ -868,7 +928,7 @@ gboolean icm_decompress_end(IcmObject *icm_object)
 gboolean icm_decompress(IcmObject *icm_object,
 							const guint flags,gpointer input,gpointer output)
 {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	ICDECOMPRESS icdecompress;
 
 	if (icm_object==NULL || input==NULL || output==NULL
@@ -897,3 +957,4 @@ gboolean icm_decompress(IcmObject *icm_object,
 	return FALSE;
 #endif
 }
+
