@@ -23,7 +23,7 @@
 #include "common.h"
 #include "bitmap.h"
 #include "chunk.h"
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 # include "dllloader/pe_image.h"
 #endif
 
@@ -32,7 +32,7 @@
 extern "C" {
 #endif
 
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 
 #define DRV_LOAD					0x0001
 #define DRV_ENABLE					0x0002
@@ -279,7 +279,7 @@ typedef struct {
 
 
 typedef struct IcmDriver_Tag {
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	PE_image *pe;
 	LONG WINAPI (*DriverProc)(DWORD dwDriverId,HANDLE hdrvr,
 										UINT msg,LONG lParam1,LONG lParam2);
@@ -301,7 +301,7 @@ typedef struct IcmObject_Tag {
 	gpointer prev;
 	gsize prev_size;
 	IcmDriver *icm_driver;
-#ifdef W32CODECDIR
+#if defined(W32CODECDIR) || defined(USE_CODEC)
 	DWORD dwDriverId;
 #endif
 } IcmObject;
