@@ -21,7 +21,7 @@
 #include "general.h"
 #include "jump.h"
 #include "prop.h"
-#include "signal.h"
+#include "sigfile.h"
 #include "size.h"
 #include "thread.h"
 #include "misc/misc.h"
@@ -180,68 +180,88 @@ void command_close(gpointer callback_data,guint callback_action,
 }
 
 
-void command_property(gpointer callback_data,guint callback_action,
+void
+command_property (gpointer callback_data, guint callback_action,
 															GtkWidget *widget)
 {
 	VmaidWindow *vmaid;
 	PropertyDialog prop;
 
-	vmaid=gtk_object_get_user_data(GTK_OBJECT(
-				gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
-					gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))));
-	prop.rate=vmaid->rate;
-	prop.scale=vmaid->scale;
-	if (vmaid->avi_edit[0]!=NULL) {
-		prop.width=avi_get_width(vmaid->avi_edit[0]);
-		prop.height=avi_get_height(vmaid->avi_edit[0]);
-		prop.bit_count=avi_get_bit_count(vmaid->avi_edit[0]);
-		prop.x_pixels_per_meter=avi_get_x_pixels_per_meter(vmaid->avi_edit[0]);
-		prop.y_pixels_per_meter=avi_get_y_pixels_per_meter(vmaid->avi_edit[0]);
-	} else {
-		prop.width=0;
-		prop.height=0;
-		prop.bit_count=0;
+	vmaid = gtk_object_get_user_data (GTK_OBJECT
+				(gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
+					gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))));
+	g_memset (&prop, 0, sizeof (PropertyDialog));
+	prop.rate = vmaid->rate;
+	prop.scale = vmaid->scale;
+	if (vmaid->avi_edit[0] != NULL)
+	  {
+		prop.priority0 = avi_get_priority (vmaid->avi_edit[0]);
+		prop.language0 = avi_get_language (vmaid->avi_edit[0]);
+		if (avi_get_name (vmaid->avi_edit[0]) != NULL)
+			prop.name0 = g_strdup (avi_get_name (vmaid->avi_edit[0]));
+		prop.width = avi_get_width (vmaid->avi_edit[0]);
+		prop.height = avi_get_height (vmaid->avi_edit[0]);
+		prop.bit_count = avi_get_bit_count (vmaid->avi_edit[0]);
+		prop.x_pixels_per_meter
+							= avi_get_x_pixels_per_meter (vmaid->avi_edit[0]);
+		prop.y_pixels_per_meter
+							= avi_get_y_pixels_per_meter (vmaid->avi_edit[0]);
+	  }
+	else
+	  {
 		prop.x_pixels_per_meter=-1;
 		prop.y_pixels_per_meter=-1;
-	}
-	if (vmaid->avi_edit[1]!=NULL) {
-		prop.samples_per_sec=avi_get_samples_per_sec(vmaid->avi_edit[1]);
-		prop.channels=avi_get_channels(vmaid->avi_edit[1]);
-		prop.bits_per_sample=avi_get_bits_per_sample(vmaid->avi_edit[1]);
-	} else {
-		prop.samples_per_sec=0;
-		prop.channels=0;
-		prop.bits_per_sample=0;
-	}
-	if (property_dialog(&prop)) {
-		vmaid->rate=prop.rate;
-		vmaid->scale=prop.scale;
-		if (vmaid->avi_edit[0]!=NULL) {
-			avi_set_rate(vmaid->avi_edit[0],prop.rate);
-			avi_set_scale(vmaid->avi_edit[0],prop.scale);
-			avi_set_width(vmaid->avi_edit[0],prop.width);
-			avi_set_height(vmaid->avi_edit[0],prop.height);
-			avi_set_bit_count(vmaid->avi_edit[0],prop.bit_count);
-			avi_set_x_pixels_per_meter(vmaid->avi_edit[0],
+	  }
+	if (vmaid->avi_edit[1] != NULL)
+	  {
+		prop.priority1 = avi_get_priority (vmaid->avi_edit[1]);
+		prop.language1 = avi_get_language (vmaid->avi_edit[1]);
+		if (avi_get_name (vmaid->avi_edit[1]) != NULL)
+			prop.name1 = g_strdup (avi_get_name (vmaid->avi_edit[1]));
+		prop.samples_per_sec = avi_get_samples_per_sec (vmaid->avi_edit[1]);
+		prop.channels = avi_get_channels (vmaid->avi_edit[1]);
+		prop.bits_per_sample = avi_get_bits_per_sample (vmaid->avi_edit[1]);
+	  }
+	if (property_dialog(&prop))
+	  {
+		vmaid->rate = prop.rate;
+		vmaid->scale = prop.scale;
+		if (vmaid->avi_edit[0] != NULL)
+		  {
+			avi_set_priority (vmaid->avi_edit[0], prop.priority0);
+			avi_set_language (vmaid->avi_edit[0], prop.language0);
+			avi_set_name (vmaid->avi_edit[0], prop.name0);
+			avi_set_rate (vmaid->avi_edit[0], prop.rate);
+			avi_set_scale (vmaid->avi_edit[0], prop.scale);
+			avi_set_width (vmaid->avi_edit[0], prop.width);
+			avi_set_height (vmaid->avi_edit[0], prop.height);
+			avi_set_bit_count (vmaid->avi_edit[0], prop.bit_count);
+			avi_set_x_pixels_per_meter (vmaid->avi_edit[0],
 													prop.x_pixels_per_meter);
-			avi_set_y_pixels_per_meter(vmaid->avi_edit[0],
+			avi_set_y_pixels_per_meter (vmaid->avi_edit[0],
 													prop.y_pixels_per_meter);
-		}
-		if (vmaid->avi_edit[1]!=NULL) {
-			avi_set_samples_per_sec(vmaid->avi_edit[1],prop.samples_per_sec);
-			avi_set_channels(vmaid->avi_edit[1],prop.channels);
-			avi_set_bits_per_sample(vmaid->avi_edit[1],prop.bits_per_sample);
-		}
-		vmaid->cursor.stream=vmaid->avi_edit[0]!=NULL?0:1;
-		vmaid->cursor.frame=0;
-		vmaid->select.stream=-1;
-		vmaid->select.frame=-1;
-		vmaid->top=0;
-		delete_list(&vmaid->undo);
-		delete_list(&vmaid->redo);
-		set_menu_bar(vmaid);
-		gtk_widget_draw(vmaid->drawing,NULL);
-	}
+			g_free (prop.name0);
+		  }
+		if (vmaid->avi_edit[1] != NULL)
+		  {
+			avi_set_priority (vmaid->avi_edit[1], prop.priority1);
+			avi_set_language (vmaid->avi_edit[1], prop.language1);
+			avi_set_name (vmaid->avi_edit[1], prop.name1);
+			avi_set_samples_per_sec (vmaid->avi_edit[1], prop.samples_per_sec);
+			avi_set_channels (vmaid->avi_edit[1], prop.channels);
+			avi_set_bits_per_sample (vmaid->avi_edit[1], prop.bits_per_sample);
+			g_free (prop.name1);
+		  }
+		vmaid->cursor.stream = vmaid->avi_edit[0] != NULL ? 0 : 1;
+		vmaid->cursor.frame = 0;
+		vmaid->select.stream = -1;
+		vmaid->select.frame = -1;
+		vmaid->top = 0;
+		delete_list (&vmaid->undo);
+		delete_list (&vmaid->redo);
+		set_menu_bar (vmaid);
+		gtk_widget_draw (vmaid->drawing, NULL);
+	  }
 }
 
 
