@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Iwamoto,Kazuki http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,12 +23,14 @@
 #include "vmaid.h"
 
 
-typedef struct {
-	guint32 rate,scale;
+typedef struct PropertyDialog_Tag {
+	guint32 rate, scale;
+	guint16 priority0, priority1, language0, language1;
+	gchar *name0, *name1;
 	/* ӥǥͭ */
-	gint32 width,height;
+	gint32 width, height;
 	guint16 bit_count;
-	gint32 x_pixels_per_meter,y_pixels_per_meter;
+	gint32 x_pixels_per_meter, y_pixels_per_meter;
 	/* ǥͭ */
 	guint32 samples_per_sec;
 	guint16 channels;
@@ -36,7 +38,8 @@ typedef struct {
 } PropertyDialog;
 
 
-gboolean property_dialog(PropertyDialog *prop);
+gboolean
+property_dialog (PropertyDialog *prop);
 
 
 #endif
