@@ -27,7 +27,8 @@
 #include "conf.h"
 #include "file.h"
 #include "general.h"
-#include "signal.h"
+#include "sigfile.h"
+#include "sigmain.h"
 #include "thread.h"
 #include "toolbar.h"
 #include "version.h"
@@ -103,8 +104,8 @@ int main(int argc,char *argv[])
 	GdkPixmap *pixmap;
 	GtkAccelGroup *accel_group;
 	GtkStyle *style;
-	GtkWidget *menubar,*toolbar,*text,*handle,*scroll,*vbox;
-	GtkWidget *sub_menu,*menu_item;
+	GtkWidget *toolbar,*text,*handle,*scroll,*vbox;
+	GtkWidget *menu_bar, *menu_shell, *menu_item;
 #ifdef USE_GTK2
 	GtkTreeSelection *select;
 	GtkTreeViewColumn *column;
@@ -323,7 +324,7 @@ int main(int argc,char *argv[])
 	gtk_item_factory_create_items(item_factory_menu,
 			sizeof(menu_items)/sizeof(GtkItemFactoryEntry),menu_items,NULL);
 	gtk_window_add_accel_group(GTK_WINDOW(window),accel_group);
-	menubar=gtk_item_factory_get_widget(item_factory_menu,"<main>");
+	menu_bar = gtk_item_factory_get_widget (item_factory_menu, "<main>");
 	/* ġС */
 #ifdef USE_GTK2
 	toolbar=gtk_toolbar_new();
@@ -404,7 +405,7 @@ int main(int argc,char *argv[])
 	vbox=gtk_vbox_new(FALSE,0);
 	handle=gtk_handle_box_new();
 	gtk_container_set_border_width(GTK_CONTAINER(handle),2);
-	gtk_container_add(GTK_CONTAINER(handle),menubar);
+	gtk_container_add (GTK_CONTAINER (handle), menu_bar);
 	gtk_box_pack_start(GTK_BOX(vbox),handle,FALSE,FALSE,0);
 	handle=gtk_handle_box_new();
 	gtk_container_set_border_width(GTK_CONTAINER(handle),2);
@@ -416,7 +417,8 @@ int main(int argc,char *argv[])
 	gtk_box_pack_end(GTK_BOX(vbox),text,FALSE,FALSE,0);
 	gtk_container_add(GTK_CONTAINER(window),vbox);
 	/* ե */
-	sub_menu=gtk_item_factory_get_widget(item_factory_menu,"<main>/File");
+	menu_shell = gtk_item_factory_get_widget (item_factory_menu,
+																"<main>/File");
 	for (i=0;i<history && history_files[i]!=NULL;i++) {
 		menu_item=gtk_menu_item_new_with_label(history_files[i]);
 		gtk_signal_connect(GTK_OBJECT(menu_item),"activate",
@@ -424,10 +426,11 @@ int main(int argc,char *argv[])
 		gtk_signal_connect(GTK_OBJECT(menu_item),"destroy",
 				GTK_SIGNAL_FUNC(signal_destroy_menu_history),history_files[i]);
 		gtk_object_set_user_data(GTK_OBJECT(menu_item),history_files[i]);
-		gtk_menu_insert(GTK_MENU(sub_menu),menu_item,i+MENUFILE-1);
+		gtk_menu_insert (GTK_MENU (menu_shell), menu_item, i + MENUFILE - 1);
 	}
 	if (i>0)
-		gtk_menu_insert(GTK_MENU(sub_menu),gtk_menu_item_new(),i+MENUFILE-1);
+		gtk_menu_insert (GTK_MENU (menu_shell), gtk_menu_item_new (),
+															i + MENUFILE - 1);
 	/*  */
 	timer_id=gtk_timeout_add(500,signal_timeout,NULL);
 
