@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2002 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2003 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,16 +20,13 @@
 #define __VMAID_H__
 
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-
-#include <gtk/gtk.h>
+#include "common.h"
 #include "avicore/avicore.h"
-#include "intl.h"
 
 
+#define VMAID_VERSION_NOTICE "Video maid "VERSION" ("VMAID_BUILD_ENVIRONMENT")"
+#define VMAID_COPYRIGHT_NOTICE "Copyright (C) 1998-2003 Kazuki IWAMOTO"
+
 #define MENUFILE 9
 
 
@@ -38,7 +35,7 @@ typedef struct VmaidCursor_Tag {
 	gint frame;					/* ߤΰ(0...) */
 } VmaidCursor;
 typedef struct VmaidHistory_Tag {
-	AviEdit *avi[2];			/* AVIԽ */
+	AviEdit *avi_edit[2];		/* AVIԽ */
 	VmaidCursor cursor,select;
 	struct VmaidHistory_Tag *next;
 } VmaidHistory;
@@ -50,13 +47,14 @@ typedef struct VmaidWindow_Tag {
 	gint width,height;			/* ե졼Υ */
 	guint rate,scale;			/* 졼ȡॹ=1ô֤Υե졼 */
 	guint timer_id;				/* ѥ */
-	AviEdit *avi[2];			/* AVIԽ */
+	AviEdit *avi_edit[2];		/* AVIԽ */
 	VmaidCursor cursor;			/* åȤΰ */
 	VmaidCursor select;			/* 򳫻ϰ,-1:ʤ */
 	VmaidHistory *undo,*redo;	/* Խ */
 	GtkWidget *drawing;			/* ΰ */
 	GtkWidget *hscroll;			/* ʿС */
 	GtkWidget *label;			/* ֤Υ٥ */
+	GtkWidget *mark;			/* ֤ԽѤߥޡ */
 	GtkWidget *menu_item;		/* ˥塼Υɥι */
 } VmaidWindow;
 
@@ -99,9 +97,13 @@ extern GtkWidget *status;				/* ơɥ */
 extern GtkPositionType n_pos;			/* Ρȥ֥åΰ */
 extern GtkItemFactory *item_factory_menu;	/* ˥塼 */
 extern GtkItemFactory *item_factory_popup;	/* ݥåץå */
+extern GtkItemFactory *item_factory_clist;	/* ݥåץå */
 extern GdkFont *system_font;			/* ե */
 extern gint system_font_height;			/* եȤι⤵ */
 extern gint system_font_ascent;			/* եȤΥȤι⤵ */
+#ifdef USE_GTK2
+extern GtkListStore *list_store;		/* ꥹ */
+#endif /* USE_GTK2 */
 extern GdkColor system_color[4];
 extern GtkTargetEntry targets[1];	/* Video maid󶡲ǽʥ쥯 */
 
